==> sal-push-2.21.1-20170313.084938-6-sources.jar: com/baijia/tianxiao/sal/push/service/impl/PushRedisServiceImpl.java <==
package com.baijia.tianxiao.sal.push.service.impl;

import com.baijia.tianxiao.constants.TianXiaoConstant;
import com.baijia.tianxiao.dal.org.dao.OrgSubAccountDao;
import com.baijia.tianxiao.dal.org.dao.TXCascadeAccountDao;
import com.baijia.tianxiao.dal.org.po.OrgSubAccount;
import com.baijia.tianxiao.dal.org.po.TXCascadeAccount;
import com.baijia.tianxiao.dal.pcAuthority.dao.TxAccountPermissionDao;
import com.baijia.tianxiao.dal.pcAuthority.po.TxAccountPermission;
import com.baijia.tianxiao.dal.push.constant.MsgSettingStatus;
import com.baijia.tianxiao.dal.push.constant.MsgUserRole;
import com.baijia.tianxiao.dal.push.dao.AccountMsgSettingDao;
import com.baijia.tianxiao.dal.push.po.AccountMsgSetting;
import com.baijia.tianxiao.dal.push.po.ConsultMessage;
import com.baijia.tianxiao.enums.CrmErrorCode;
import com.baijia.tianxiao.exception.BussinessException;
import com.baijia.tianxiao.redis.AbstractBaseRedisDao;
import com.baijia.tianxiao.sal.common.api.ConsulterAPIService;
import com.baijia.tianxiao.sal.push.constant.UserRoleEnum;
import com.baijia.tianxiao.sal.push.service.PushRedisService;
import com.baijia.tianxiao.sal.push.service.UserCacheService;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by liuxp on 16/5/29.
 */
@Slf4j
@Service
public class PushRedisServiceImpl extends AbstractBaseRedisDao<String, Object> implements PushRedisService {

    private static final String LAST_PUSH_NO_HASH = "TX#LAST#PUSH#NO#HASH";
    private static final String CONSULT_USER_AVATAR_HASH = "TX#CONSULT#USER#AVATAR#HASH";
    private static final String ORG_ACCOUNT_TYPE = "TX#ORG#ACCOUNT#TYPE#HASH";
    private static final String ORG_MESSAGE_KEY_PREFIX = "TX#ORG#MESSAGE#";
    private static final long ORG_MESSAGE_KEY_EXPIRE = 5 * 60;

    @Autowired
    private TXCascadeAccountDao cascadeAccountDao;
    @Autowired
    private OrgSubAccountDao orgSubAccountDao;
    @Autowired
    private AccountMsgSettingDao accountMsgSettingDao;
    @Autowired
    private TxAccountPermissionDao permissionDao;
    @Autowired
    private ConsulterAPIService consulterAPIService;
    @Autowired
    private UserCacheService cacheService;

    @Override
    public Integer getConsultUserCount(final Long orgId) {
        return redisTemplate.execute(new RedisCallback<Integer>() {
            @Override
            public Integer doInRedis(RedisConnection connection) throws DataAccessException {
                log.info("[Redis] Query start.");
                connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                RedisSerializer<String> serializer = getRedisSerializer();
                byte[] key = serializer.serialize(LAST_PUSH_NO_HASH);
                byte[] field = serializer.serialize(orgId.toString());

                byte[] value = connection.hGet(key, field);
                log.info("[Redis] Query success===");
                if (value == null) {
                    return 0;
                } else {
                    int result = Integer.parseInt(serializer.deserialize(value));
                    log.info("[Redis] Query Result =" + result);
                    return result;
                }
            }
        });
    }

    @Override
    public void addConsultUserCount(final Long orgId) {
        redisTemplate.execute(new RedisCallback<Object>() {
            @Override
            public Object doInRedis(RedisConnection connection) throws DataAccessException {
                connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                RedisSerializer<String> serializer = getRedisSerializer();
                byte[] key = serializer.serialize(LAST_PUSH_NO_HASH);
                byte[] field = serializer.serialize(orgId.toString());
                connection.hIncrBy(key, field, 1);
                log.info("[Redis] Update success. orgId=" + orgId);
                return null;
            }
        });
    }

    @Override
    public List<Integer> getCascadeIds(Long orgId) {
        List<Integer> ids = new ArrayList<>();
        List<TXCascadeAccount> cascadeAccounts = cascadeAccountDao.getTXCascadeAccountListByOrgId(orgId);

        if ((cascadeAccounts != null) && (cascadeAccounts.size() > 0)) {
            for (TXCascadeAccount account : cascadeAccounts) {
                ids.add(account.getId());
            }

            // 105代表咨询权限，参考TXPermissionConst
            List<TxAccountPermission> permissions = permissionDao.selectPermission(ids, 105L);
            log.info("[Switch] permissions cascadeIds=" + permissions);
            if (permissions != null) {
                for (TxAccountPermission tp : permissions) {
                    if (tp.getPType() == 1) {
                        log.info("[Switch] No permission,cascadeId=" + tp.getUId());
                        ids.remove(tp.getUId());
                    }
                }
            }

            ids.add(0, 0);

            List<AccountMsgSetting> settings = accountMsgSettingDao.getAccountMsgSettingList(orgId, ids);
            List<Integer> closeIds = new ArrayList<>();

            for (AccountMsgSetting setting : settings) {
                if (setting.getMsgSwitch() == MsgSettingStatus.CLOSE.getValue()) {
                    closeIds.add(setting.getCascadeId());
                }
            }

            log.info("[Switch] all cascadeIds=" + ids);
            log.info("[Switch] close cascadeIds=" + closeIds);
            ids.removeAll(closeIds);
            return ids;
        }
        return ids;
    }

    @Override
    public void setOrgAccountType(final Long orgId, final Integer cascadeId, final int type) {
        redisTemplate.execute(new RedisCallback<Object>() {
            @Override
            public Object doInRedis(RedisConnection connection) throws DataAccessException {
                connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                RedisSerializer<String> serializer = getRedisSerializer();
                byte[] key = serializer.serialize(ORG_ACCOUNT_TYPE);
                byte[] field = serializer.serialize(getKey(orgId, cascadeId));
                connection.hIncrBy(key, field, type);
                log.info("[Redis] Update success. orgId={},cascadeId={},type={}", orgId, cascadeId, type);
                return null;
            }
        });
    }

    @Override
    public int getOrgAccountType(final Long orgId, final Integer cascadeId) {

        Integer accountType = redisTemplate.execute(new RedisCallback<Integer>() {
            @Override
            public Integer doInRedis(RedisConnection connection) throws DataAccessException {
                connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                RedisSerializer<String> serializer = getRedisSerializer();
                byte[] key = serializer.serialize(ORG_ACCOUNT_TYPE);
                byte[] field = serializer.serialize(getKey(orgId, cascadeId));
                RedisSerializer redisSerializer = redisTemplate.getDefaultSerializer();
                byte[] value = connection.hGet(key, field);
                log.info("[Redis] Query success.");
                if (value != null) {
                    return null;
                } else {
                    return (Integer) redisSerializer.deserialize(value);
                }
            }
        });

        if (accountType == null) {
            accountType = getAccountTypeFromDb(orgId, cascadeId);
            setOrgAccountType(orgId, cascadeId, accountType);
        }

        return accountType;
    }

    private int getAccountTypeFromDb(Long orgId, Integer cascadeId) {
        if ((cascadeId == null) || (cascadeId == 0)) {
            OrgSubAccount subAccount = orgSubAccountDao.getMasterByOrgId(orgId.intValue());
            log.debug("OrgId=" + orgId + ";OrgSubAccount=" + subAccount);
            if (subAccount != null) {
                return MsgUserRole.HEADER.getValue();
            } else {
                return MsgUserRole.BRANCH_HEADER.getValue();
            }
        } else {
            TXCascadeAccount cascadeAccount = cascadeAccountDao.getById(cascadeId);
            if (!MsgUserRole.isSubOrg(cascadeAccount.getAccountType())) {
                throw new BussinessException(CrmErrorCode.ORG_ACCOUNT_TYPE_ERROR);
            }
            return cascadeAccount.getAccountType();
        }
    }

    private String getKey(Long orgId, Integer cascadeId) {
        if (cascadeId == null) {
            cascadeId = 0;
        }
        return orgId + "_" + cascadeId;
    }

    @Override
    public void setAvatar(final long consultUserId, final String url) {
        try {
            redisTemplate.execute(new RedisCallback<Object>() {
                @Override
                public Object doInRedis(RedisConnection connection) throws DataAccessException {
                    connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                    RedisSerializer<String> serializer = getRedisSerializer();
                    byte[] key = serializer.serialize(CONSULT_USER_AVATAR_HASH);
                    byte[] field = serializer.serialize(String.valueOf(consultUserId));
                    byte[] value = serializer.serialize(url);
                    connection.hSet(key, field, value);
                    return null;
                }
            });
        } catch (Exception e) {
            log.error("[Redis] Exception {}", e);
        }
    }

    @Override
    public void setAvatar(final long consultUserId) {
        try {
            String avatarUrl = consulterAPIService.getConsultAvatarUrl(consultUserId);
            setAvatar(consultUserId, avatarUrl);
        } catch (Exception e) {
            log.error("[Redis] Exception {}", e);
        }

    }

    @Override
    public String getAvatar(final long consultUserId) {
        String avatar = null;
        try {
            avatar = redisTemplate.execute(new RedisCallback<String>() {
                @Override
                public String doInRedis(RedisConnection connection) throws DataAccessException {
                    connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                    RedisSerializer<String> serializer = getRedisSerializer();
                    byte[] key = serializer.serialize(CONSULT_USER_AVATAR_HASH);
                    byte[] field = serializer.serialize(String.valueOf(consultUserId));
                    byte[] value = connection.hGet(key, field);
                    if (value != null) {
                        return serializer.deserialize(value);
                    } else {
                        return null;
                    }
                }
            });
        } catch (Exception e) {
            log.error("[Redis] Exception {}", e);
        }

        return avatar;
    }

    @Override
    public void addMessage(final ConsultMessage message) {
        try {
            redisTemplate.execute(new RedisCallback<Void>() {
                @Override
                public Void doInRedis(RedisConnection connection) throws DataAccessException {
                    log.info("[Redis] Cache message,orgId={},msgId={}",message.getOrgId(),message.getId());
                    connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                    RedisSerializer<String> serializer = getRedisSerializer();
                    byte[] key = serializer.serialize(getOrgMessageKey(message.getOrgId()));
                    Gson gson = new Gson();
                    String msgJson = gson.toJson(message);
                    byte[] value = serializer.serialize(msgJson);
                    connection.sAdd(key, value);
                    connection.expire(key, ORG_MESSAGE_KEY_EXPIRE);
                    return null;
                }
            });
        }catch (Exception e){
            log.error("Redis Exception {}",e);
        }
    }
    
    @Override
    public List<ConsultMessage> getMessageList(final Long orgId, final Long cascadeId) {
        List<ConsultMessage> result = new ArrayList<>();
        try {
            result = redisTemplate.execute(new RedisCallback<List<ConsultMessage>>() {
                @Override
                public List<ConsultMessage> doInRedis(RedisConnection connection) throws DataAccessException {
                    connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                    List<ConsultMessage> messages = new ArrayList<>();
                    RedisSerializer<String> serializer = getRedisSerializer();
                    byte[] key = serializer.serialize(getOrgMessageKey(orgId));
                    Gson gson = new Gson();
                    Set<byte[]> bytes = connection.sMembers(key);
                    if (bytes != null && bytes.size() > 0) {
                        for (byte[] byteArr : bytes) {
                            String str = serializer.deserialize(byteArr);
                            ConsultMessage message = gson.fromJson(str, ConsultMessage.class);
                            if (isMyDialog(cascadeId, message)) {
                                messages.add(message);
                            }
                        }
                    }
                    return messages;
                }
            });
        }catch (Exception e){
            log.error("Redis Exception {}",e);
        }
        return result;
    }
    
    private boolean isMyDialog(long cascadeId, ConsultMessage message) {
        Integer orgNumber = cacheService.getOrgNumber(message.getOrgId());
        long sendId = message.getSenderId();
        long receiverId = message.getReceiverId();
        if (cascadeId == 0) {
            cascadeId = orgNumber.longValue();
        }
        if (cascadeId == sendId || cascadeId == receiverId) {
            return true;
        } else {
            return false;
        }
    }
    
    private String getOrgMessageKey(Long orgId) {
        return ORG_MESSAGE_KEY_PREFIX + orgId;
    }
}

==> sal-push-2.44.0-20171016.123219-21-sources.jar: com/baijia/tianxiao/sal/push/service/impl/PushRedisServiceImpl.java <==
package com.baijia.tianxiao.sal.push.service.impl;

import com.baijia.tianxiao.constants.TianXiaoConstant;
import com.baijia.tianxiao.dal.org.dao.OrgSubAccountDao;
import com.baijia.tianxiao.dal.org.dao.TXCascadeAccountDao;
import com.baijia.tianxiao.dal.org.po.OrgSubAccount;
import com.baijia.tianxiao.dal.org.po.TXCascadeAccount;
import com.baijia.tianxiao.dal.pcAuthority.dao.TxAccountPermissionDao;
import com.baijia.tianxiao.dal.pcAuthority.po.TxAccountPermission;
import com.baijia.tianxiao.dal.push.constant.MsgSettingStatus;
import com.baijia.tianxiao.dal.push.constant.MsgUserRole;
import com.baijia.tianxiao.dal.push.dao.AccountMsgSettingDao;
import com.baijia.tianxiao.dal.push.po.AccountMsgSetting;
import com.baijia.tianxiao.dal.push.po.ConsultMessage;
import com.baijia.tianxiao.enums.CrmErrorCode;
import com.baijia.tianxiao.enums.RedisKeyEnums;
import com.baijia.tianxiao.exception.BussinessException;
import com.baijia.tianxiao.redis.AbstractBaseRedisDao;
import com.baijia.tianxiao.sal.common.api.ConsulterAPIService;
import com.baijia.tianxiao.sal.push.constant.UserRoleEnum;
import com.baijia.tianxiao.sal.push.service.PushRedisService;
import com.baijia.tianxiao.sal.push.service.UserCacheService;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by liuxp on 16/5/29.
 */
@Slf4j
@Service
public class PushRedisServiceImpl extends AbstractBaseRedisDao implements PushRedisService {

//    private static final String LAST_PUSH_NO_HASH = "TX#LAST#PUSH#NO#HASH";
//    private static final String CONSULT_USER_AVATAR_HASH = "TX#CONSULT#USER#AVATAR#HASH";
//    private static final String ORG_ACCOUNT_TYPE = "TX#ORG#ACCOUNT#TYPE#HASH";
//    private static final String ORG_MESSAGE_KEY_PREFIX = "TX#ORG#MESSAGE#";
    private static final long ORG_MESSAGE_KEY_EXPIRE = 5 * 60;

    @Autowired
    private TXCascadeAccountDao cascadeAccountDao;
    @Autowired
    private OrgSubAccountDao orgSubAccountDao;
    @Autowired
    private AccountMsgSettingDao accountMsgSettingDao;
    @Autowired
    private TxAccountPermissionDao permissionDao;
    @Autowired
    private ConsulterAPIService consulterAPIService;
    @Autowired
    private UserCacheService cacheService;

    @Override
    public Integer getConsultUserCount(final Long orgId) {
        return redisTemplate.execute(new RedisCallback<Integer>() {
            @Override
            public Integer doInRedis(RedisConnection connection) throws DataAccessException {
                log.info("[Redis] Query start.");
                connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                RedisSerializer<String> serializer = getRedisSerializer();
                byte[] key = serializer.serialize(RedisKeyEnums.CRM.LAST_PUSH_NO_HASH.getRedisKey());
                byte[] field = serializer.serialize(orgId.toString());

                byte[] value = connection.hGet(key, field);
                log.info("[Redis] Query success===");
                if (value == null) {
                    return 0;
                } else {
                    int result = Integer.parseInt(serializer.deserialize(value));
                    log.info("[Redis] Query Result =" + result);
                    return result;
                }
            }
        });
    }

    @Override
    public void addConsultUserCount(final Long orgId) {
        redisTemplate.execute(new RedisCallback<Object>() {
            @Override
            public Object doInRedis(RedisConnection connection) throws DataAccessException {
                connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                RedisSerializer<String> serializer = getRedisSerializer();
                byte[] key = serializer.serialize(RedisKeyEnums.CRM.LAST_PUSH_NO_HASH.getRedisKey());
                byte[] field = serializer.serialize(orgId.toString());
                connection.hIncrBy(key, field, 1);
                log.info("[Redis] Update success. orgId=" + orgId);
                return null;
            }
        });
    }

    @Override
    public List<Integer> getCascadeIds(Long orgId) {
        List<Integer> ids = new ArrayList<>();
        List<TXCascadeAccount> cascadeAccounts = cascadeAccountDao.getTXCascadeAccountListByOrgId(orgId);

        if ((cascadeAccounts != null) && (cascadeAccounts.size() > 0)) {
            for (TXCascadeAccount account : cascadeAccounts) {
                ids.add(account.getId());
            }

            // 105代表咨询权限，参考TXPermissionConst
            List<TxAccountPermission> permissions = permissionDao.selectPermission(ids, 105L);
            log.info("[Switch] permissions cascadeIds=" + permissions);
            if (permissions != null) {
                for (TxAccountPermission tp : permissions) {
                    if (tp.getPType() == 1) {
                        log.info("[Switch] No permission,cascadeId=" + tp.getUId());
                        ids.remove(tp.getUId());
                    }
                }
            }

            ids.add(0, 0);

            List<AccountMsgSetting> settings = accountMsgSettingDao.getAccountMsgSettingList(orgId, ids);
            List<Integer> closeIds = new ArrayList<>();

            for (AccountMsgSetting setting : settings) {
                if (setting.getMsgSwitch() == MsgSettingStatus.CLOSE.getValue()) {
                    closeIds.add(setting.getCascadeId());
                }
            }

            log.info("[Switch] all cascadeIds=" + ids);
            log.info("[Switch] close cascadeIds=" + closeIds);
            ids.removeAll(closeIds);
            return ids;
        }
        return ids;
    }

    @Override
    public void setOrgAccountType(final Long orgId, final Integer cascadeId, final int type) {
        redisTemplate.execute(new RedisCallback<Object>() {
            @Override
            public Object doInRedis(RedisConnection connection) throws DataAccessException {
                connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                RedisSerializer<String> serializer = getRedisSerializer();
                byte[] key = serializer.serialize(RedisKeyEnums.CRM.ORG_ACCOUNT_TYPE.getRedisKey());
                byte[] field = serializer.serialize(getKey(orgId, cascadeId));
                connection.hIncrBy(key, field, type);
                log.info("[Redis] Update success. orgId={},cascadeId={},type={}", orgId, cascadeId, type);
                return null;
            }
        });
    }

    @Override
    public int getOrgAccountType(final Long orgId, final Integer cascadeId) {

        Integer accountType = redisTemplate.execute(new RedisCallback<Integer>() {
            @Override
            public Integer doInRedis(RedisConnection connection) throws DataAccessException {
                connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                RedisSerializer<String> serializer = getRedisSerializer();
                byte[] key = serializer.serialize(RedisKeyEnums.CRM.ORG_ACCOUNT_TYPE.getRedisKey());
                byte[] field = serializer.serialize(getKey(orgId, cascadeId));
                RedisSerializer redisSerializer = redisTemplate.getDefaultSerializer();
                byte[] value = connection.hGet(key, field);
                log.info("[Redis] Query success.");
                if (value != null) {
                    return null;
                } else {
                    return (Integer) redisSerializer.deserialize(value);
                }
            }
        });

        if (accountType == null) {
            accountType = getAccountTypeFromDb(orgId, cascadeId);
            setOrgAccountType(orgId, cascadeId, accountType);
        }

        return accountType;
    }

    private int getAccountTypeFromDb(Long orgId, Integer cascadeId) {
        if ((cascadeId == null) || (cascadeId == 0)) {
            OrgSubAccount subAccount = orgSubAccountDao.getMasterByOrgId(orgId.intValue());
            log.debug("OrgId=" + orgId + ";OrgSubAccount=" + subAccount);
            if (subAccount != null) {
                return MsgUserRole.HEADER.getValue();
            } else {
                return MsgUserRole.BRANCH_HEADER.getValue();
            }
        } else {
            TXCascadeAccount cascadeAccount = cascadeAccountDao.getById(cascadeId);
            if (!MsgUserRole.isSubOrg(cascadeAccount.getAccountType())) {
                throw new BussinessException(CrmErrorCode.ORG_ACCOUNT_TYPE_ERROR);
            }
            return cascadeAccount.getAccountType();
        }
    }

    private String getKey(Long orgId, Integer cascadeId) {
        if (cascadeId == null) {
            cascadeId = 0;
        }
        return orgId + "_" + cascadeId;
    }

    @Override
    public void setAvatar(final long consultUserId, final String url) {
        try {
            redisTemplate.execute(new RedisCallback<Object>() {
                @Override
                public Object doInRedis(RedisConnection connection) throws DataAccessException {
                    connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                    RedisSerializer<String> serializer = getRedisSerializer();
                    byte[] key = serializer.serialize(RedisKeyEnums.CRM.CONSULT_USER_AVATAR_HASH.getRedisKey());
                    byte[] field = serializer.serialize(String.valueOf(consultUserId));
                    byte[] value = serializer.serialize(url);
                    connection.hSet(key, field, value);
                    return null;
                }
            });
        } catch (Exception e) {
            log.error("[Redis] Exception {}", e);
        }
    }

    @Override
    public void setAvatar(final long consultUserId) {
        try {
            String avatarUrl = consulterAPIService.getConsultAvatarUrl(consultUserId);
            setAvatar(consultUserId, avatarUrl);
        } catch (Exception e) {
            log.error("[Redis] Exception {}", e);
        }

    }

    @Override
    public String getAvatar(final long consultUserId) {
        String avatar = null;
        try {
            avatar = redisTemplate.execute(new RedisCallback<String>() {
                @Override
                public String doInRedis(RedisConnection connection) throws DataAccessException {
                    connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                    RedisSerializer<String> serializer = getRedisSerializer();
                    byte[] key = serializer.serialize(RedisKeyEnums.CRM.CONSULT_USER_AVATAR_HASH.getRedisKey());
                    byte[] field = serializer.serialize(String.valueOf(consultUserId));
                    byte[] value = connection.hGet(key, field);
                    if (value != null) {
                        return serializer.deserialize(value);
                    } else {
                        return null;
                    }
                }
            });
        } catch (Exception e) {
            log.error("[Redis] Exception {}", e);
        }

        return avatar;
    }

    @Override
    public void addMessage(final ConsultMessage message) {
        try {
            redisTemplate.execute(new RedisCallback<Void>() {
                @Override
                public Void doInRedis(RedisConnection connection) throws DataAccessException {
                    log.info("[Redis] Cache message,orgId={},msgId={}",message.getOrgId(),message.getId());
                    connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                    RedisSerializer<String> serializer = getRedisSerializer();
                    byte[] key = serializer.serialize(getOrgMessageKey(message.getOrgId()));
                    Gson gson = new Gson();
                    String msgJson = gson.toJson(message);
                    byte[] value = serializer.serialize(msgJson);
                    connection.sAdd(key, value);
                    connection.expire(key, ORG_MESSAGE_KEY_EXPIRE);
                    return null;
                }
            });
        }catch (Exception e){
            log.error("Redis Exception {}",e);
        }
    }


    @Override
    public List<ConsultMessage> getMessageList(final Long orgId, final Long cascadeId) {
        List<ConsultMessage> result = new ArrayList<>();
        try {
            result = redisTemplate.execute(new RedisCallback<List<ConsultMessage>>() {
                @Override
                public List<ConsultMessage> doInRedis(RedisConnection connection) throws DataAccessException {
                    connection.select(TianXiaoConstant.TX_PV_REDIS_DB);
                    List<ConsultMessage> messages = new ArrayList<>();
                    RedisSerializer<String> serializer = getRedisSerializer();
                    byte[] key = serializer.serialize(getOrgMessageKey(orgId));
                    Gson gson = new Gson();
                    Set<byte[]> bytes = connection.sMembers(key);
                    if (bytes != null && bytes.size() > 0) {
                        for (byte[] byteArr : bytes) {
                            String str = serializer.deserialize(byteArr);
                            ConsultMessage message = gson.fromJson(str, ConsultMessage.class);
                            if (isMyDialog(cascadeId, message)) {
                                messages.add(message);
                            }
                        }
                    }
                    return messages;
                }
            });
        }catch (Exception e){
            log.error("Redis Exception {}",e);
        }
        return result;
    }

    private boolean isMyDialog(long cascadeId, ConsultMessage message) {
        Integer orgNumber = cacheService.getOrgNumber(message.getOrgId());
        long sendId = message.getSenderId();
        long receiverId = message.getReceiverId();
        if (cascadeId == 0) {
            cascadeId = orgNumber.longValue();
        }
        if (cascadeId == sendId || cascadeId == receiverId) {
            return true;
        } else {
            return false;
        }
    }

    private String getOrgMessageKey(Long orgId) {
        return RedisKeyEnums.CRM.ORG_MESSAGE_KEY_PREFIX.getRedisKey() + orgId;
    }
}
